use rocket_contrib::json::Json;
use rocket::{Response, State};
use rocket::http::Status;

use crate::config::Config;
use crate::DbConn;
use crate::models::errors::{ErrorResponse, make_500};
use crate::models::users::{
    UserInfo,
    LocalUser,
    CreateUserRequest,
    CreateUserZoneRequest,
    AuthClaims,
    AuthTokenRequest,
    AuthTokenResponse
};


#[post("/users/me/token", data = "<auth_request>")]
pub async fn create_auth_token(
    conn: DbConn,
    config: State<'_, Config>,
    auth_request: Json<AuthTokenRequest>
) -> Result<Json<AuthTokenResponse>, ErrorResponse> {

    let user_info = conn.run(move |c| {
        LocalUser::get_user_by_creds(c, &auth_request.username, &auth_request.password)
    }).await?;

    let token = AuthClaims::new(&user_info, config.web_app.token_duration)
        .encode(&config.web_app.secret)
        .map_err(make_500)?;

    Ok(Json(AuthTokenResponse { token }))
}

#[post("/users", data = "<user_request>")]
pub async fn create_user<'r>(conn: DbConn, user_request: Json<CreateUserRequest>) -> Result<Response<'r>, ErrorResponse> {
    // TODO: Check current user if any to check if user has permission to create users (with or without role)
    conn.run(|c| {
        LocalUser::create_user(&c, user_request.into_inner())
    }).await?;

    Response::build()
        .status(Status::Created)
        .ok()
}

#[post("/users/<user_id>/zones", data = "<user_zone_request>")]
pub async fn create_user_zone<'r>(
    conn: DbConn,
    user_info: Result<UserInfo, ErrorResponse>,
    user_id: String,
    user_zone_request: Json<CreateUserZoneRequest>
) -> Result<Response<'r>, ErrorResponse>{
    user_info?.check_admin()?;

    conn.run(move |c| {
        let user_info = LocalUser::get_user_by_uuid(&c, user_id)?;
        user_info.add_zone(&c, user_zone_request.into_inner())
    }).await?;

    Response::build()
        .status(Status::Created)
        .ok()
}
